/* filename:    dprior.g
** description: The program generates draws from the prior of the 
**              DSGE model
** created:     12/14/01
*/

library pgraph, user;
cls;

block_n    = 10;
block_size = 10;
mhrun      = "0";   /* NOTE: we use mhrun = 0 to denote prior */

#include c:\projects\active\dsgeprio\gausscgg\pathspec.g;
#include c:\projects\active\dsgeprio\gausscgg\varspec.g

modelid   = ftos(mspec,"%*.*lf",1,0);
lmodel    = "d" $+ modelid; 

priorfile = lmodel $+ "pri.out";
load path = ^priopath prior[npara,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];

/* Initialize Output files
*/
opath = parapath $+ "\\mhrun" $+ mhrun $+"\\";
opara = opath $+ modelid $+ "0101" $+ "mh";
create fhpara=^opara with PARAM, npara, 8;

/* Loop to generate parameter draws
*/
block_it = 1;
para_acpt = 0;

draws_total = 0;
draws_valid = 0;

do until block_it > block_n;

   locate 1,1;
   "Processing Block" block_it;

   parasim  = zeros(block_size,npara);
   validsim = zeros(block_size,1);

   i = 1; 
   do until i > npara;
   
      if pshape[i] == 1; /* BETA Prior */
         a = (1-pmean[i])*pmean[i]^2/pstdd[i]^2 - pmean[i];
         b = a*(1/pmean[i] - 1);
         parasim[.,i] = rndbeta(block_size,1,a,b);
      
      elseif pshape[i] == 2; /* GAMMA PRIOR */
         b = pstdd[i]^2/pmean[i];
         a = pmean[i]/b;
         parasim[.,i] = b*rndgam(block_size,1,a);
      
      elseif pshape[i] == 3; /* GAUSSIAN PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = a + b*rndn(block_size,1);

      elseif pshape[i] == 4; /* INVGAMMA PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = sqrt( b*a^2/sumc( (rndn(b,block_size))^2 ) );

      elseif pshape[i] == 5; /* uniform prior */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = a + (b-a)*rndu(block_size,1);

      elseif pshape[i] == 0; /* No Prior - Fixed */
         a = pmean[i];
         parasim[.,i] = a*ones(block_size,1);

      endif;

      i = i+1;
   endo; 

   i = 1;
   do until i > block_size;

      {TTT, RRR, valid} = dsgesolv( parasim[i,.]', mspec);
      if valid == 1;
         validsim[i,1] = 1;
      else;
         validsim[i,1] = 0;
      endif;

      i = i+1;

   endo;

   draws_total = draws_total + block_size;
   draws_valid = draws_valid + sumc(validsim);

   parasim = delif(parasim, (validsim .EQ 0) );

   wr = writer(fhpara,parasim);
   block_it = block_it + 1;

endo;

closeall fhpara;

"Fraction of valid draws";
100*draws_valid/draws_total;

#include c:\projects\active\dsgeprio\gausscgg\procs\dsgemod.src