/* filename:    predplt1.g
** description: The program plots predictive distributions
**              baseline and policy intervention
**
**              baseline = dashed
**              intervention = dots
**              actual = solid
**
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;

#include c:\projects\active\dsgeprio\Gausscgg\pathspec.g
#include c:\projects\active\dsgeprio\Gausscgg\varspec.g

mhrun    = "1";
mhrunpol = "1p";

modelid = ftos(mspec,"%*.*lf",1,0);

/* Fix Time Period
*/
Tind = 1;
T0   = Tgrid[Tind,1];
Tend = Tgrid[Tind,2];

#include c:\projects\active\dsgeprio\Gausscgg\procs\loaddat2.g   

load path = ^datapath yyfut, nobsfut;
ti        =  seqa(1,1,nobsfut);

/* Compute Cumulative Growth Rates
*/
yycfut = zeros(nobsfut,nvar);
yycfut[1,.] = yyfut[1,.];
t=2;
do until t > nobsfut;
   yycfut[t,.] = yycfut[t-1,.] + yyfut[t,.];
   t = t+1;
endo;

lamgridex = 0 | lamgrid;

/* Initialize Graph
*/
graphset;
begwind;
margin(0,0,0.2,0.2);
window(rows(lamgridex),3,0);
fonts("microb");
   _ptitlht = 0.3;
   _paxht = 0.3;
   _pnumht = 0.25;
   _pcolor = 2 ;
   _pltype = {1 1 2 2 6};
   _protate = 0; 
   _plctrl = 0;
   _plwidth = 5.5;
   _psymsiz = 5;


lamindex = 1;
do until lamindex > rows(lamgridex);

    lamind = lamindex - 1;

    /******************************************
    ** Open files with Policy Loss  summaries
    */

    if lamind < 10;
      llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
    else;
      llam = ftos(lamind,"%*.*lf",2,0);
    endif;

    if Tind < 10;
      lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
    else;
      lsamp = ftos(Tind,"%*.*lf",2,0);
    endif;

    lpath  = predpath $+ "\\mhrun" $+ mhrun    $+ "\\";
    lpathp = predpath $+ "\\mhrun" $+ mhrunpol $+ "\\";

    if lamind == 0;
      lvys  = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "ys"; 
      lvysp = lpathp $+ "v" $+ modelid $+ "01" $+ lsamp $+ "ys"; 
      lvps  = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "ps"; 
      lvpsp = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "ps"; 
      lvrs  = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "rs"; 
      lvrsp = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "rs"; 

     
    else;
      lvys  = lpath  $+ "v" $+ modelid $+ llam $+ lsamp $+ "ys"; 
      lvysp = lpathp $+ "v" $+ modelid $+ llam $+ lsamp $+ "ys"; 
      lvps  = lpath  $+ "v" $+ modelid $+ llam $+ lsamp $+ "ps"; 
      lvpsp = lpathp $+ "v" $+ modelid $+ llam $+ lsamp $+ "ps"; 
      lvrs  = lpath  $+ "v" $+ modelid $+ llam $+ lsamp $+ "rs"; 
      lvrsp = lpathp $+ "v" $+ modelid $+ llam $+ lsamp $+ "rs"; 

    endif;

    /* Read Posterior Simulation Summary File:
    ** DSGE model
    */
    open fhvys  = ^lvys for read;
    open fhvysp = ^lvysp for read;
    open fhvps  = ^lvps for read;
    open fhvpsp = ^lvpsp for read;
    open fhvrs  = ^lvrs for read;
    open fhvrsp = ^lvrsp for read;

    vys_m     = readr(fhvys,1);
    vys_s     = readr(fhvys,1);
    vys_hpd   = readr(fhvys,2);

    vysp_m     = readr(fhvysp,1);
    vysp_s     = readr(fhvysp,1);
    vysp_hpd   = readr(fhvysp,2);

    vps_m     = readr(fhvps,1);
    vps_s     = readr(fhvps,1);
    vps_hpd   = readr(fhvps,2);

    vpsp_m     = readr(fhvpsp,1);
    vpsp_s     = readr(fhvpsp,1);
    vpsp_hpd   = readr(fhvpsp,2);

    vrs_m     = readr(fhvrs,1);
    vrs_s     = readr(fhvrs,1);
    vrs_hpd   = readr(fhvrs,2);

    vrsp_m     = readr(fhvrsp,1);
    vrsp_s     = readr(fhvrsp,1);
    vrsp_hpd   = readr(fhvrsp,2);


    closeall fhvys, fhvysp, fhvps, fhvpsp, fhvrs, fhvrsp;  

    titlestry = "Output Growth (Cumulative), LAMBDA = " 
                $+ ftos(lamgridex[lamindex],"%*.*lf",6,2);

    titlestrp = "Inflation (Cumulative), LAMBDA = " 
                $+ ftos(lamgridex[lamindex],"%*.*lf",6,2);

    titlestrr = "Interest Rate, LAMBDA = " 
                $+ ftos(lamgridex[lamindex],"%*.*lf",6,2);

    title(titlestry);
    xy(ti, (vys_hpd' ~ vysp_hpd' ~ yycfut[.,1]));
    nextwind;

    title(titlestrp);
    xy(ti, (vps_hpd' ~ vpsp_hpd' ~ yycfut[.,2]));
    nextwind;

    title(titlestrr);
    xy(ti, (vrs_hpd' ~ vrsp_hpd' ~ yyfut[.,3]));
    nextwind;

    lamindex = lamindex+1;

endo;

endwind;