/* filename:    predplt4.g
** description: The program plots predictive distributions
**              baseline and policy intervention
**
**              baseline = dashed
**              intervention = dots
**              actual = solid
**
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;

#include c:\projects\active\dsgeprio\Gausscgg\pathspec.g
#include c:\projects\active\dsgeprio\Gausscgg\varspec.g

mhrun    = "1";
mhrunpol = "1p";

traj1 = 14;

modelid = ftos(mspec,"%*.*lf",1,0);

/* Fix Time Period
*/
Tind = 1;
T0   = Tgrid[Tind,1];
Tend = Tgrid[Tind,2];

#include c:\projects\active\dsgeprio\Gausscgg\procs\loaddat2.g   

load path = ^datapath yyfut, nobsfut;
ti        =  seqa(1,1,nobsfut);

lamgridex = 0 | lamgrid;

/* Compute Cumulative Growth Rates
*/
yycfut = zeros(nobsfut,nvar);
yycfut[1,.] = yyfut[1,.];
t=2;
do until t > nobsfut;
   yycfut[t,.] = yycfut[t-1,.] + yyfut[t,.];
   t = t+1;
endo;


/* Initialize Graph
*/
graphset;
begwind;
margin(0,0,0.2,0.2);
window(rows(lamgridex),3,0);
fonts("microb");
   _ptitlht = 0.3;
   _paxht = 0.3;
   _pnumht = 0.25;
   _pcolor = 2 ;
   _pltype = {1 2 6};
   _protate = 0; 
   _plctrl = 0;
   _plwidth = 5.5;
   _psymsiz = 5;

lamindex = 1;
do until lamindex > rows(lamgridex);

    lamind = lamindex - 1;

    /******************************************
    ** Open files with Policy Loss  summaries
    */

    if lamind < 10;
      llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
    else;
      llam = ftos(lamind,"%*.*lf",2,0);
    endif;

    if Tind < 10;
      lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
    else;
      lsamp = ftos(Tind,"%*.*lf",2,0);
    endif;

    lpath  = predpath $+ "\\mhrun" $+ mhrun    $+ "\\";
    lpathp = predpath $+ "\\mhrun" $+ mhrunpol $+ "\\";

    if lamind == 0;
      lvys  = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "ym"; 
      lvysp = lpathp $+ "v" $+ modelid $+ "01" $+ lsamp $+ "ym"; 
      lvps  = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "pm"; 
      lvpsp = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "pm"; 
      lvrs  = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "rm"; 
      lvrsp = lpath  $+ "v" $+ modelid $+ "01" $+ lsamp $+ "rm"; 
     
    else;
      lvys  = lpath  $+ "v" $+ modelid $+ llam $+ lsamp $+ "ym"; 
      lvysp = lpathp $+ "v" $+ modelid $+ llam $+ lsamp $+ "ym"; 
      lvps  = lpath  $+ "v" $+ modelid $+ llam $+ lsamp $+ "pm"; 
      lvpsp = lpathp $+ "v" $+ modelid $+ llam $+ lsamp $+ "pm"; 
      lvrs  = lpath  $+ "v" $+ modelid $+ llam $+ lsamp $+ "rm"; 
      lvrsp = lpathp $+ "v" $+ modelid $+ llam $+ lsamp $+ "rm"; 

    endif;

    /* Read Posterior Simulation Summary File:
    ** DSGE model
    */
    open fhvys  = ^lvys for read;
    open fhvysp = ^lvysp for read;
    open fhvps  = ^lvps for read;
    open fhvpsp = ^lvpsp for read;
    open fhvrs  = ^lvrs for read;
    open fhvrsp = ^lvrsp for read;

    ys_grid   = readr(fhvys,1)';
    ys_pdf    = readr(fhvys,1)';

    ysp_grid    = readr(fhvysp,1)';
    ysp_pdf     = readr(fhvysp,1)';

    ps_grid     = readr(fhvps,1)';
    ps_pdf      = readr(fhvps,1)';

    psp_grid    = readr(fhvpsp,1)';
    psp_pdf     = readr(fhvpsp,1)';

    rs_grid     = readr(fhvrs,1)';
    rs_pdf      = readr(fhvrs,1)';

    rsp_grid    = readr(fhvrsp,1)';
    rsp_pdf     = readr(fhvrsp,1)';

    closeall fhvys, fhvysp, fhvps, fhvpsp, fhvrs, fhvrsp;  

    titlestry = "Mean(Output Growth), LAMBDA = " 
                $+ ftos(lamgridex[lamindex],"%*.*lf",6,2);

    titlestrp = "Mean(Inflation), LAMBDA = " 
                $+ ftos(lamgridex[lamindex],"%*.*lf",6,2);

    titlestrr = "Mean(Interest Rate), LAMBDA = " 
                $+ ftos(lamgridex[lamindex],"%*.*lf",6,2);

    ys_pdflow  = 0;
    ys_pdfhigh = maxc( ys_pdf | ysp_pdf );    
    ys_actual  = stdc(yyfut[traj1:rows(yyfut),1]);

    ps_pdflow  = 0;
    ps_pdfhigh = maxc( ps_pdf | psp_pdf );    
    ps_actual  = stdc(yyfut[traj1:rows(yyfut),2]);

    rs_pdflow  = 0;
    rs_pdfhigh = maxc( rs_pdf | rsp_pdf );    
    rs_actual  = stdc(yyfut[traj1:rows(yyfut),3]);

    title(titlestry);
    _pline = ( 1 ~ 6 ~ ys_actual ~ ys_pdflow ~ ys_actual ~ ys_pdfhigh ~ 4 ~ 1 );
    xy(ys_grid, ys_pdf ~ ysp_pdf);
    nextwind;

    title(titlestrp);
    _pline = ( 1 ~ 6 ~ ps_actual ~ ps_pdflow ~ ps_actual ~ ps_pdfhigh ~ 4 ~ 1 );
    xy(ps_grid, ps_pdf ~ psp_pdf);
    nextwind;

    title(titlestrr);
    _pline = ( 1 ~ 6 ~ rs_actual ~ rs_pdflow ~ rs_actual ~ rs_pdfhigh ~ 4 ~ 1 );
    xy(rs_grid, rs_pdf ~ rsp_pdf);
    nextwind;

    lamindex = lamindex+1;

endo;

endwind;