/* filename: loaddat2.g
** description: reads time series data from ASCII data set into
** Gauss variable SERIES
** created:  05/03/00
*/

/* Time series, extracted 21/11/01
** Gross Domestic Product (SAAR, Bil.Chn.1996$), Bureau of Economic Analysis
** CPI-U: All Items (SA, 1982-84=100), Bureau of Labor Statistics
** Federal Funds Rate 1,2
** Quarterly Data from 1947:I to 2001:III
*/

datafile = datapath $+ "//data4.txt";
load data_q[219,4]=^datafile;

/* Compute growth rates: from 1947:II to 2001:III, 218 observations
*/
series =  100*( ln(data_q[2:rows(data_q),1:2])-ln(data_q[1:rows(data_q)-1,1:2]) ) ~
              ( data_q[2:rows(data_q),3] );
          
/* Eliminate Observations for which the Federal Funds Rate is not available
** -> 189 observations, starting 1954:III
*/
series = series[30:rows(series),.];
          
nobs   = rows(series);
ti     = seqa(1954.5,0.25,nobs);

/* Note: we need at least T0 = nlags to initialize the lags of the VAR
*/

YYall     = series[1+T0:nobs,.];
XXall     = ones(nobs-T0, 1+nvar*nlags);
ti        = ti[1+T0:nobs,.];


i = 1;
do until i > nlags;
   XXall[.,2+(i-1)*nvar:1+i*nvar] = series[T0-i+1:nobs-i,.];
   i = i+1;
endo;

YY     = YYall[1:Tend-T0,.];
XX     = XXall[1:Tend-T0,.];
if Tend-T0+1 LE rows(YYall);
   YYfut  = YYall[Tend-T0+1:rows(YYall),.];
else;
   YYfut  = zeros(1,nvar);
endif;

XXpred = 1 ~ YYall[Tend-T0,.];
if nlags > 1;
     XXpred = XXpred ~ XXall[Tend-T0,2:1+nvar*(nlags-1)];
endif;

YYYY = YY'*YY;
XXYY = XX'*YY;
XXXX = XX'*XX;

nobs = rows(YY);
nobsfut = minc( 89 |rows(yyfut) );

YYfut = YYfut[1:nobsfut,.];

/***********************************************
**           Specify Minnesota Prior
*/

/* Prior for Sigma
*/
nu_min = 3;    
H_min = nu_min*eye(nvar);


/* Prior for C
*/
CClag1 = zeros(nvar,nvar);
CClag1[3,3] = 1;

if nlags == 1;
   CCt_min = zeros(nvar,1) ~ CClag1;
else;
   CCt_min = zeros(nvar,1) ~ CClag1 ~ zeros(nvar,nvar*(nlags-1));
endif;

CC_min  = CCt_min';
C_min   = vec(CC_min);

/* run univariate AR(1)'s
*/
ar1_sig = zeros(nvar,1);
i = 1;
do until i > nvar;
   yyar1 = yy[2:nobs,i];
   xxar1 = ones(nobs-1,1) ~ yy[1:nobs-1,i];
   ar1_sig[i] = (yyar1'*yyar1 - yyar1'*xxar1*inv(xxar1'*xxar1)*xxar1'*yyar1)/(nobs-1);
   i = i+1;
endo;

/* Prior variance for intercept 
*/
intcpt_v0 = 5; 

/* Prior variance for autoregressive
** coefficients
*/
SIG   = ones(nvar,1) .*. ar1_sig';
SIG   = SIG .* ( (25 ~ 100 ~ 100) | ( 100 ~ 25 ~ 100 ) | ( 100 ~ 100 ~ 25 ) );

VC_mininv  = ones(nvar,1)/intcpt_v0;
i = 1;
do until i > nlags;
   VC_mininv = VC_mininv~SIG.*i^2;
   i = i+1;
endo;

VC_mininv = eye(nvar*(1+nvar*nlags)).*vec(VC_mininv');


save path = ^datapath nobs, xxxx, yyyy, xxyy, yyfut, xxpred, nobsfut, yy , xx;
