/* filename:    irfpmom.g
** description: The program computes posterior mean and confidence bands
**              for the VAR impulse response functions
** created:     11/06/01
*/

/********************************************************
** Open files with IRF Draws
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

lpath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";

if lamind == 0;
   ldraw  = lpath $+ lmodel $+ "01" $+ lsamp $+ "ir"; 
   odraws = opath $+ lmodel $+ "01" $+ lsamp $+ "is"; 
else;
   ldraw  = lpath $+ lmodel $+ llam $+ lsamp $+ "ir"; 
   odraws = opath $+ lmodel $+ llam $+ lsamp $+ "is"; 
endif;

open fhdraw = ^ldraw for read;


drawrow = readr( fhdraw, nburn); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, nburn);

/* Part 1: Compute the mean of x(i) and x^2(i)
*/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);

eofloop = 0;
nblock  = 100;
ndraws  = 0;

do until eofloop; 

   drawblock = readr( fhdraw, nblock );

   drawmean  = drawmean    + sumc(drawblock)';
   drawsqmean = drawsqmean + sumc(drawblock^2)';

   ndraws = ndraws + rows(drawblock);

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;
drawsqmean = drawsqmean/ndraws;
drawstdd   = sqrt(drawsqmean - (drawmean)^2);



/* Part 2: HPD Interval
*/
drawrow     = seekr( fhdraw, nburn); 
drawci      = zeros(2,drawdim);

j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, nburn); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, nblock );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);
         
   endo;

   drawci[.,j] = hpdint(drawcol,hpdprob);
   j = j+1;

endo;
clear drawcol;

create fhdraws = ^odraws with DRAWSUM, drawdim, 8;
wr=writer(fhdraws, real(drawmean | drawstdd | drawci) );
closeall fhdraws; 


/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;
