/* filename:    marghess.g
** description: The program computes the hessian at the posterior mode 
** created:     09/20/01
*/

/********************************************************
** Load sufficient statistics for dataset
*/
load path=^datapath yyyy, xxxx, xxyy, nobs;

/********************************************************
** Load information for prior distribution
*/

modelid   = ftos(mspec,"%*.*lf",1,0);
lmodel    = "d" $+ modelid;

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

priorfile = lmodel $+ "pri.out";
load path = ^priopath prior[npara,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];


/******************************************************** 
** Load Posterior Mode Estimates
*/
lpara = postpath $+ "\\" $+ modelid $+ llam $+lsamp $+ "pm";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;
load path = ^priopath para_names;
priolhmode = fcn(para);

/* Optional Screen Output
*/
if __output == 1;
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);

"";
"Prior*Likelihood at Mode";
priolhmode;
cls;
endif;


/* Compute Hessian, element by element, fine tune with dxscale
*/
comphess:

ndx = 6;
dx =  exp(-seqa(6,2,ndx));
hessian = zeros( npara, npara );
gradx = zeros(ndx,1);
grady = zeros(ndx,1);
gradxy = zeros(ndx,1);
hessdiag = zeros(ndx,1);


/* Compute Diagonal elements first
*/
seli = 1;
do until seli > npara;     
     
     i=1;
     do until i > ndx;
      paradx = para;
      parady = para;
      paradx[seli] = paradx[seli] + dx[i]*dxscale[seli];
      parady[seli] = parady[seli] - dx[i]*dxscale[seli];
      paradxdy = paradx;
      paradxdy[seli] = paradxdy[seli] - dx[i]*dxscale[seli];
      fx  = fcn(para);
      fdx = fcn(paradx);
      fdy = fcn(parady);
      fdxdy = fcn(paradxdy);
      gradx[i] = -( fx - fdx )/ (dx[i]*dxscale[seli]);
      grady[i] = ( fx - fdy )/ (dx[i]*dxscale[seli]);
      gradxy[i] = -(fx -fdxdy)/ sqrt( (dx[i]*dxscale[seli])^2 + (dx[i]*dxscale[seli])^2 );
      hessdiag[i] = -( 2*fx - fdx - fdy)/(dx[i]*dxscale[seli])^2; 
      hessdiag[i] = -( fx - fdx - fdy + fdxdy )/(dx[i]*dx[i]*dxscale[seli]*dxscale[seli]);
      i = i+1;
     endo;

     if -(hessdiag[3]+hessdiag[4]) > 0;
        hessian[seli,seli] = -0.5*(hessdiag[3]+hessdiag[4]);
     elseif meanc(-hessdiag) > 0;
        hessian[seli,seli] = meanc(-hessdiag[2:5]);
     else;
        hessian[seli,seli] = maxc(-hessdiag[2:5]);
     endif;

     /* Optional Screen Output
     */
     if __output == 1;
        locate 1,1;
        "Hessian Element    (" seli seli ")";
        "Numerical Values";
        hessdiag';
       "Value Used:" hessian[seli,seli];
       "Press key to continue";
       kpress = keyw;
     endif;   
     
   seli = seli+1;
endo;

/* Now compute off-diagonal elements
** Make sure that correlations are between -1 and 1
** errorij contains the index of elements that are invalid
*/
errorij = 0~0~0;

seli = 1;
do until seli == npara;
   selj = seli+1;
   do until selj > npara;
          
     i=1;
     do until i > ndx;
      paradx = para;
      parady = para;
      paradx[seli] = paradx[seli] + dx[i]*dxscale[seli];
      parady[selj] = parady[selj] - dx[i]*dxscale[selj];
      paradxdy = paradx;
      paradxdy[selj] = paradxdy[selj] - dx[i]*dxscale[selj];
      fx  = fcn(para);
      fdx = fcn(paradx);
      fdy = fcn(parady);
      fdxdy = fcn(paradxdy);
      gradx[i] = -( fx - fdx )/ (dx[i]*dxscale[seli]);
      grady[i] = ( fx - fdy )/ (dx[i]*dxscale[selj]);
      gradxy[i] = -(fx -fdxdy)/ sqrt( (dx[i]*dxscale[selj])^2 + (dx[i]*dxscale[seli])^2 );
      hessdiag[i] = -( 2*fx - fdx - fdy)/(dx[i]*dxscale[seli])^2; 
      hessdiag[i] = -( fx - fdx - fdy + fdxdy )/(dx[i]*dx[i]*dxscale[seli]*dxscale[selj]);
      i = i+1;
     endo;

     hessian[seli,selj] = -0.5*(hessdiag[3]+hessdiag[4]);
     
     if ( hessian[seli,seli] == 0 ) or (hessian[selj,selj] == 0);
        corrij = 0;
     else;
        corrij = hessian[seli,selj]/sqrt(hessian[seli,seli]*hessian[selj,selj]);
     endif;

     if (corrij < -1) OR (corrij > 1);
        hessian[seli,selj]=0;
        errorij = errorij |(seli~selj~corrij);
     elseif (abs(corrij) < 0.01);
        hessian[seli,selj]=0;
     endif;   
     hessian[selj,seli] = hessian[seli,selj];

     /* Optional Screen Output
     */
     if __output == 1;
        locate 1,1;
        "Hessian Element    (" seli selj ")";
        "Numerical Values";
        hessdiag';
        "Value Used:" hessian[seli,selj];
        "Press key to continue";
        kpress = keyw;

     endif;       
     
     selj=selj+1;
   endo;
   seli = seli+1;
endo;

/* Initialize Output files
*/
ohess = postpath $+ "\\" $+ modelid $+ llam $+lsamp $+ "hs";
create fhhess=^ohess with hess, npara, 8;
wr    = writer(fhhess,hessian[1:npara,1:npara]);
closeall fhhess;

/*******************************************************************************
*/

/* Load Hessian, compute penalty
*/
evalhess:

lhess = postpath $+ "\\" $+ modelid $+llam $+ lsamp $+ "hs";
open fhhess = ^lhess for read;
HHm   = readr( fhhess,npara );
closeall fhhess;

if mspec == 2;
   rankHHm = npara-2;
else;
   rankHHm = npara;
endif;

/* Create Inverse by Singular Value Decomposition
*/
{u , s, v} = svd1(HHM);
invHHMdet = 1;

i = 1;
do until i > npara;
   if i > rankHHM;
      s[i,i] = 0;
   else;
      s[i,i]    = 1/s[i,i];
      invHHMdet = invHHMdet*s[i,i];
   endif;
   i = i+1;
endo;

invHHM  = u*s*u';
sigmult = u*sqrt(s);

screen off;
output file = ^outstr on;

penalt = (rankHHM/2)*ln(2*pi) + 0.5*ln(invHHMdet);
laplaceapprox = priolhmode+penalt;
"Prior*Likelihood " priolhmode;
"Post Mode Penalty" penalt;
"Laplace Approx   " laplaceapprox;
"";

output file = ^outstr off;
screen on;

/* Initialize Output files
*/
omult = postpath $+ "\\" $+ modelid $+ llam $+ lsamp $+ "mu";
create fhmult=^omult with MULT, npara, 8;
wr = writer(fhmult,sigmult);

closeall fhmult;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = fcn(para);
local lnpY, lnprio;

/* likelihood 
*/
  lnpy = marglh( para,lambda );

/* Evaluate the Prior density
*/      
  lnprio = priodens( para, pmean, pstdd, pshape);

retp(real(lnpY+lnprio));  
endp;
