/* filename:    prioproc.g
** created:     11/05/01
*/

proc(4) = varprio(para);
/* Global nvar, nlags, mspec, npara
**
** description: The program converts the cross moment matrices for the DSGE prior
*/

local retcode,valid,
      TTT,RRR,
      yyyyd,xxxxd,xxyyd,
      nstate,
      sigr, sigg, sigz, lngamm, lnpist, lnr2st,
      ZZ,DD,HH,QQ,VV,
      GAMM0,GA0,Gl,TTl,
      l,r;

     retcode = 1;

     {TTT, RRR, valid} = dsgesolv( para, mspec);
          
     if valid < 1;
        retcode = 0;

        yyyyd = zeros(nvar,nvar);
        xxyyd = zeros(1+nlags*nvar,nvar);
        xxxxd = zeros(1+nlags*nvar,1+nlags*nvar);
       
        goto eodsgeprio;
     endif;

     sigr = para[11];
     sigg = para[12];
     sigz = para[13];
     lngamm = para[1];
     lnpist = para[2];
     lnr2st = para[3]+para[2];

     nstate = rows(TTT);

     /* create system matrices for state space model
     */

     /* composition of state vector
     ** [y,inf,r2,r2star,g,z,xi_y,xi_infl,y-1]'
     */

     ZZ = zeros(nvar,nstate);
     ZZ[1,1] = 1; ZZ[1,9] = -1; ZZ[1,6]=1;
     ZZ[2,2] = 1; 
     ZZ[3,3] = 4;
     DD = lngamm | lnpist | 4*lnr2st;

     HH = zeros(nvar,nvar);
     
     QQ = eye(nvar).* (sigr^2|sigg^2|sigz^2);

     VV = zeros(nvar,nvar);

     /* Compute nlags autocovariances
     */

     /* Initialize Autocovariances
     */
     GAMM0 = zeros(nvar^2,nlags+1);

     /*
     GA0  = inv(eye(nstate^2) - (TTT.*.TTT))*vec(RRR*QQ*RRR');
     GA0  = reshape(GA0,nstate,nstate);
     */
     
     GA0 = dlyap(TTT,RRR*QQ*RRR');

     Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;

     GAMM0[.,1] = vecr(Gl);
 
     TTl = TTT;
     l=1;
     do until l > nlags;
      Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
      GAMM0[.,l+1] = vecr(Gl);           
      TTl = TTl*TTT;
      l = l+1; 
     endo;

     /* Create limit cross product matrices
     */
     yyyyd = reshape(GAMM0[.,1],nvar,nvar) + DD*DD';
     
     xxyyd = DD;
     l = 1;
     do until l > nlags;
        xxyyd = xxyyd ~ (reshape(GAMM0[.,l+1],nvar,nvar) + DD*DD');
        l = l+1;
     endo;
     xxyyd = xxyyd';
     
     xxxxd = zeros(1+nlags*nvar,1+nlags*nvar);
     xxxxd[1,1] = 1;
     xxxxd[1,2:1+nlags*nvar] = ones(1,nlags).*.DD';
     xxxxd[2:1+nlags*nvar,1] = ones(nlags,1).*.DD;
     
     r = 1;
     do until r>nlags;
        l = r;
        do until l>nlags;
           xxxxd[1+nvar*(r-1)+1:1+nvar*r , 1+nvar*(l-1)+1:1+nvar*l ] = reshape(GAMM0[.,l-r+1],nvar,nvar)+DD*DD';
           xxxxd[1+nvar*(l-1)+1:1+nvar*l , 1+nvar*(r-1)+1:1+nvar*r ] = reshape(GAMM0[.,l-r+1],nvar,nvar)'+DD*DD';
           l = l+1;
        endo;
        r = r+1;
     endo;
     
     eodsgeprio:
     
     retp(yyyyd,xxyyd,xxxxd,retcode);
endp;
     

proc(1) = marglh(para,lambda);

/* global
**     yyyy, xxxx, xxyy, nobs,
**     nlags, nvar,
*/

local yyyyd, xxxxd, xxyyd,
      yyyyc, xxxxc, xxyyc,
      retcode,
      k, pyt, pytd, nobsc, lnkapt, lnkaptd,
      SIG_tilde, SIG_stard,i;

    /* Compute cross product matrices for DSGE dummy observations
    */
    {yyyyd, xxyyd, xxxxd, retcode} = varprio(para);

    if retcode == 0;
       pYT = -1E10;
       pYTd = -1E-10;
       goto eodsgepost;
    endif;

    yyyyc = yyyy + lambda*nobs*yyyyd;
    xxyyc = xxyy + lambda*nobs*xxyyd;
    xxxxc = xxxx + lambda*nobs*xxxxd;

    nobsc = nobs + nobs*lambda;    
    k     = 1+nlags*nvar;
    
    /* Compute OLS estimates for dummy observations
    */
    SIG_stard = ( YYYYd -XXYYd'*inv(XXXXd)*XXYYd );

    lnkaptd = 0.5*nvar*(nobs*lambda-k)*ln(2);
    i = 1;
    do until i > nvar;
       lnkaptd = lnkaptd + lngam(0.5*(lambda*nobs-k+1-i));
       i=i+1;
    endo;
            
    pYtd  = lnkaptd  - 0.5*(lambda*nobs-k)*ln(det(lambda*nobs*Sig_stard)) - 0.5*nvar*ln(det(lambda*nobs*XXXXd));

    /* Compute OLS estimates for actual and dummy observations
    */
    SIG_tilde = ( YYYYc -XXYYc'*inv(XXXXc)*XXYYc )/nobsc;

    lnkapt = -0.5*nobs*nvar*ln(2*pi) + 0.5*nvar*(nobsc-k)*ln(2);
    i = 1;
    do until i > nvar;
       lnkapt = lnkapt + lngam(0.5*(nobsc-k+1-i));
       i = i+1;
    endo;
            
    pYt  = lnkapt  - 0.5*(nobsc-k)*ln(det(nobsc*Sig_tilde)) - 0.5*nvar*ln(det(XXXXc)) - pYtd;
    
    eodsgepost:
    
    retp(pYt);
endp;
