/* filename:    vac.g
** description: The program computes autocovariances
**              for the VAR 
** created:     11/06/01
*/

/********************************************************
** Open files with Parameter Draws
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

lpath = parapath $+ "\\mhrun" $+ mhrun $+ "\\";

if lamind == 0;
   lpara = lpath $+ lmodel $+ "01" $+ lsamp; 
else;
   lpara = lpath $+ lmodel $+ llam $+ lsamp; 
endif;

open fhpara = ^lpara for read;

/* Define some Constants
*/
nvarpara  = nvar*(1+nvar*nlags) + nvar*(nvar+1)/2;

/* Initialize Output files
*/
opath     = resupath $+ "\\mhrun" $+ mhrun $+"\\";
ovarac    = opath $+ omodel $+ llam $+ lsamp $+ "ac";

create fhvarac = ^ovarac with AC, nvar+nvar^2, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 1;
nsim    = 0;

do until eofloop; 

   /* Read blocks of size nread
   */
   varparasim = readr( fhpara, nblock );
   nread      = rows(varparasim);
   acsim      = zeros(nread,nvar+nvar^2);

   locate 3,1;
   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhpara);
   
   j = 1;
   do until j > nread;

      /* Create Matrices CC and Sig
      */
      SIG_sim = xpnd(varparasim[j,nvar*(1+nvar*nlags)+1:nvarpara]');
      CCt_sim = reshape(varparasim[j,1:nvar*(1+nvar*nlags)]',nvar,1+nvar*nlags);
      CC_sim  = CCt_sim';
      
      /* Compute MA representation from companion form VAR
      */ 
      if nlags == 1;
        CCtilde  =  CCt_sim[.,1+1:1+nlags*nvar];
        CC0tilde = CCt_sim[.,1];
        SIGtilde = SIG_sim;
      else;
        CCtilde  =  CCt_sim[.,1+1:1+nlags*nvar] |
                   (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
        CC0tilde = CCt_sim[.,1] | zeros(nvar*(nlags-1),1);
        SIGtilde = (SIG_sim ~ zeros(nvar, nvar*(nlags-1)) )|
                   zeros(nvar*(nlags-1),nvar*nlags);
      endif;

      AC0tilde   = reshape( ( inv(eye(rows(CCtilde)^2) - (CCtilde.*.CCtilde))*vec(Sigtilde) ),rows(CCtilde),rows(CCtilde) );
      yybartilde = inv(eye(rows(CCtilde))-CCtilde)*CC0tilde;
      
      if nlags == 1;
         Msel = eye(nvar);
      else;
         Msel    = eye(nvar) | zeros((nlags-1)*nvar,nvar) ;
      endif;
      
      AC0   = Msel'*AC0tilde*Msel;
      yybar = Msel'*yybartilde;
      
      acsim[j,.] = yybar' ~ vec(AC0)';
            
      j = j+1;
  endo;

  wr = writer(fhvarac,acsim);
  loopct = loopct + 1;

endo;

closeall fhvarac, fhpara;
