/* filename:    vddrawh.g
** description: The program computes conditional means
**              of PHI from the VAR conditional 
**              on the DSGE model responses
** created:     11/06/01
*/

/********************************************************
** Load sufficient statistics for dataset
*/
load path=^datapath yyyy, xxxx, xxyy, nobs;

/********************************************************
** Open files with Parameter Draws
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

modelid = ftos(mspec,"%*.*lf",1,0);
omodel  = "vd" $+ modelid;
lpath = parapath $+ "\\mhrun" $+ mhrun $+ "\\";

if lamind == 0;
   lpara = lpath $+ modelid $+ "01" $+ lsamp $+ "mh"; 
else;
   lpara = lpath $+ modelid $+ llam $+ lsamp $+ "mh"; 
endif;

open fhpara = ^lpara for read;

/* Define some Constants
*/
nvarpara  = nvar*(1+nvar*nlags);
nvarparah = hmax*nvar*(1+nvar*nlags);

/* Initialize Output files
*/
opath      = parapath $+ "\\mhrun" $+ mhrun $+"\\";
ovarparah  = opath $+ omodel $+ llam $+ lsamp $+ "h";

create fhvarparah = ^ovarparah with VARPARAH, nvarpara, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 1;
nsim    = 0;

varparahmean = zeros(1,nvarparah);

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr( fhpara, nblock );
   nread   = rows(parasim);

   locate 3,1;
   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhpara);

   varparahsim = zeros(nread,nvarparah);
   
   j = 1;
   do until j > rows(parasim);

      /* For each para, draw a CC and Sig for the VAR
      */
      {yyyyd, xxyyd, xxxxd, retcode} = varprio(parasim[j,.]');
      
      yyyyc = yyyy + lambda*nobs*yyyyd;
      xxyyc = xxyy + lambda*nobs*xxyyd;
      xxxxc = xxxx + lambda*nobs*xxxxd;

      nobsc = nobs + nobs*lambda;    
      k     = 1+nlags*nvar;
    
      /* Compute OLS estimates
      */
      CC_hat  = inv(XXXXc)*XXYYc;
      C_hat   = vec(CC_hat);
      SIG_hat = ( YYYYc -XXYYc'*inv(XXXXc)*XXYYc )/nobsc;

      val = 0;
      do until val == 1;

         /* draw from the marginal posterior of SIG
         */
         H = nobsc*Sig_hat;
         Z = chol(inv(H))'*rndn(nvar,nobsc-k);
         Sig_sim = inv(Z*Z');

         /* draw from the conditional posterior of C
         */
         VC     = Sig_sim.*.inv(XXXXc);
         C_sim  = C_hat + chol(VC)'*rndn(nvar*k,1);
         CCt_sim= reshape(C_sim,nvar,k);

         /* Check for stationarity
         */
         if nlags == 1;
            CCtilde =  CCt_sim[.,1+1:k];
         else;
            CCtilde =  CCt_sim[.,1+1:k] |
                      (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
         endif;
         laCCtil = eig(CCtilde);
         if maxc(abs(laCCtil)) < 1; 
            val = 1;
         endif;

      endo; 

      if nlags == 1;
        CCtilde =  CCt_sim[.,1+1:k];
        mutilde =  CCt_sim[.,1];
      else;
        CCtilde =  CCt_sim[.,1+1:k] |
                   (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
        mutilde =  CCt_sim[.,1] | zeros(nvar*(nlags-1),1) ;
      endif;

      varparah = vec(C_sim)';
      
      if hmax > 1;
         hind = 2;
         CCtildeh = CCtilde;
         mutildeh = mutilde;
         
         do until hind > hmax;
         
            CCtildeh = CCtilde*CCtildeh;
            mutildeh = mutilde + CCtildeh*mutildeh;
            CCth_sim = mutildeh[1:nvar,.] ~ CCtildeh[1:nvar,.];
            varparah = varparah ~ vec(CCth_sim')';
         
            hind = hind + 1;
         endo;
         
      endif;
         
      varparahsim[j,.] = varparah;      
            
      j = j+1;
  endo;

  nsim = nsim + nread;
  varparahmean = varparahmean + sumc(varparahsim)';      

  loopct = loopct + 1;

endo;

varparahmean = varparahmean/nsim;
varparahmean = reshape(varparahmean,hmax,nvarpara);

wr = writer(fhvarparah,varparahmean);

screen off;
output file = ^outstr on;
varparahmean;
output file = ^outstr off;
screen on;

closeall fhvarparah, fhpara;
