/* filename:    vdirfid2.g
** description: The program computes generates the identification of
**              the VAR impulse response functions
**              functions for the DSGE model
** created:     11/06/01
*/

/********************************************************
** Open files with Parameter Draws
*/

modelid = ftos(mspec,"%*.*lf",1,0);

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

lpath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";

if lamind == 0;
   lvirf = lpath $+ "v" $+ modelid $+ "01" $+ lsamp $+ "i0"; 
   lmirf = lpath $+ "d" $+ modelid $+ "01" $+ lsamp $+ "i0"; 
else;
   lvirf = lpath $+ "v" $+ modelid $+ llam $+ lsamp $+ "i0"; 
   lmirf = lpath $+ "d" $+ modelid $+ llam $+ lsamp $+ "i0"; 
endif;

open fhvirf = ^lvirf for read;
open fhmirf = ^lmirf for read;

/* Initialize Output files
*/
opath  = resupath $+ "\\mhrun" $+ mhrun $+"\\";
omirf  = opath $+ "d" $+ modelid $+ llam $+ lsamp $+ "ir";
ovirf  = opath $+ "v" $+ modelid $+ llam $+ lsamp $+ "ir";

create fhomirf = ^omirf with IRF, nvar^2*nirf, 8;
create fhovirf = ^ovirf with IRF, nvar^2*nirf, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 1;

do until eofloop; 

   /* Read blocks of size nsim
   */
   mirf0sim = readr( fhmirf, nblock );
   virf0sim = readr( fhvirf, nblock );
   
   nread   = rows(mirf0sim);

   locate 3,1;
   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhvirf);
   eofloop = 1;

   mirfsim = zeros(nread,nvar^2*nirf);
   virfsim = zeros(nread,nvar^2*nirf);
   
   j = 1;
   do until j > nread;

      AAirf = reshape(mirf0sim[j,.]',nvar^2,nirf)'; /* DSGE Model Response */
      BBirf = reshape(virf0sim[j,.]',nvar^2,nirf)'; /* VAR Response */
      
      AA0 = reshape(AAirf[1,.],nvar,nvar)';
      {AArot,AAtr} = qqr(AA0');
      
      AAtr  = AAtr';
      AArot = AArot';
      
      AAirftr = AAirf;
      BBirftr = BBirf*(AArot.*.eye(nvar));
      
      mirfsim[j,.] = vec(AAirftr)';
      virfsim[j,.] = vec(BBirftr)';
      
      j = j+1;
  endo;
      
  /* Write Output
  */
  wr = writer(fhomirf,mirfsim);
  wr = writer(fhovirf,virfsim);

  loopct = loopct + 1;

endo;

closeall fhomirf, fhovirf, fhmirf, fhvirf;
