/* filename:    virf.g
** description: The program computes Cholesky IRFs
**              for the VAR 
** created:     11/06/01
*/

/********************************************************
** Open files with Parameter Draws
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

lpath = parapath $+ "\\mhrun" $+ mhrun $+ "\\";

if lamind == 0;
   lpara = lpath $+ lmodel $+ "01" $+ lsamp; 
else;
   lpara = lpath $+ lmodel $+ llam $+ lsamp; 
endif;

open fhpara = ^lpara for read;

/* Define some Constants
*/
nvarpara  = nvar*(1+nvar*nlags) + nvar*(nvar+1)/2;

/* Initialize Output files
*/
opath      = resupath $+ "\\mhrun" $+ mhrun $+"\\";
ovarirf    = opath $+ omodel $+ llam $+ lsamp $+ "i0";

create fhvarirf = ^ovarirf with IRF, nvar^2*nirf, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 1;
nsim    = 0;

do until eofloop; 

   /* Read blocks of size nread
   */
   varparasim = readr( fhpara, nblock );
   nread      = rows(varparasim);
   irfsim     = zeros(nread,nvar^2*nirf);

   locate 3,1;
   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhpara);
   
   j = 1;
   do until j > nread;

      /* Create Matrices CC and Sig
      */
      SIG_sim = xpnd(varparasim[j,nvar*(1+nvar*nlags)+1:nvarpara]');
      CCt_sim = reshape(varparasim[j,1:nvar*(1+nvar*nlags)]',nvar,1+nvar*nlags);
      CC_sim  = CCt_sim';
      SIG_chol = chol(SIG_sim)';
      
      /* Compute MA representation from companion form VAR
      */
      CCtilde =  CCt_sim[.,1+1:1+nlags*nvar] |
                (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));

      CCpower = eye(nvar*nlags);
      Msel    = eye(nvar) | zeros((nlags-1)*nvar,nvar) ;
      
      BBsim       = zeros(nirf,nvar^2);
      BBcsim      = zeros(nirf,nvar^2);
      BBsim[1,.]  = vec(Msel'*CCpower*Msel*SIG_chol)';
      BBcsim[1,.] = vec(Msel'*CCpower*Msel*SIG_chol)';
      
      t = 2;
      do until t > nirf;
      
         CCpower = CCtilde*CCpower;
         BBsim[t,.]  = vec(Msel'*CCpower*Msel*SIG_chol)';
         BBcsim[t,.] = BBcsim[t-1,.] + vec(Msel'*CCpower*Msel*SIG_chol)';
         t = t+1;

      endo;
      
      irfsim[j,.] = BBcsim[.,1]' ~ /* output <- policy */
                    BBcsim[.,2]' ~ /* cumulative inflation <- policy */
                    BBsim[.,3]'  ~ /* interest rate <- policy */                    
                    BBcsim[.,4]' ~ /* output <- g-shock */
                    BBcsim[.,5]' ~ /* cumulative inflation <- g-shock */
                    BBsim[.,6]'  ~ /* interest rate <- g-shock */                    
                    BBcsim[.,7]' ~ /* output <- z-shock */
                    BBcsim[.,8]' ~ /* cumulative inflation <- z-shock */
                    BBsim[.,9]';   /* interest rate <- z-shock */  
      
      BBsim = reshape(irfsim[j,.],nvar^2,nirf)';
            
      j = j+1;
  endo;



  wr = writer(fhvarirf,irfsim);
  loopct = loopct + 1;

endo;

closeall fhvarirf, fhpara;
