/* filename:    vmdraw.g
** description: The program generates draws from the VAR
**              with Minnesota Prior
** created:     11/06/01
*/

/********************************************************
** Load sufficient statistics for dataset
*/
load path=^datapath yyyy, xxxx, xxyy, nobs, yy, xx;

/********************************************************
** Open files with Parameter Draws
*/

if zetind < 10;
   llam = "0" $+ ftos(zetind,"%*.*lf",1,0);
else;
   llam = ftos(zetind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

omodel = "vm";

/* Define some Constants
*/
nvarpara  = nvar*(1+nvar*nlags) + nvar*(nvar+1)/2;
nvarparah = hmax*nvar*(1+nvar*nlags);

/* Initialize Output files
*/
opath      = parapath $+ "\\mhrun" $+ mhrun $+"\\";
ovarpara   = opath $+ omodel $+ llam $+ lsamp;
ovarstat   = opath $+ omodel $+ llam $+ lsamp $+ "st";
ovarparah  = opath $+ omodel $+ llam $+ lsamp $+ "h";

create fhvarpara  = ^ovarpara with VARPARA, nvarpara, 8;
create fhvarparah = ^ovarparah with VARPARAH, nvarparah/hmax, 8;
create fhvarstat  = ^ovarstat with STAT, 2, 8;

/* Initialization
*/ 
blockind   = 1;
totaldraws = 0;

k = 1+nlags*nvar;

varparahmean = zeros(1,nvarparah);
    
/* Compute OLS estimates
*/
CC_hat  = inv(XXXX)*XXYY;
C_hat   = vec(CC_hat);
SIG_hat = ( YYYY -XXYY'*inv(XXXX)*XXYY )/nobs;


/* Initialize the Gibbs Sampler
*/
Sigi_sim = inv(Sig_hat);


/* Minnesota Prior Matrices
** C_min, CC_min, CCt_min,  VC_mininv
** H_min, vu_min
*/

do until blockind > nblock; 

   locate 3,1;
   "Block     " blockind;
   "Blocksize " nsim;

   varparasim  = zeros(nsim,nvarpara);
   varparahsim = zeros(nsim,nvarparah);
   statsim     = zeros(nsim,2);
   
   j = 1;
   do until j > nsim;     
         
     val = 0;
     do until val == 1;


       /* Draw C_sim
       */
       VC_tili = VC_mininv*zeta + Sigi_sim.*.XXXX;
       VC_til  = inv(VC_tili);
       C_til   = VC_til*(zeta*VC_mininv*C_min + (Sigi_sim.*.XXXX)*C_hat);
       CCt_til = reshape(C_til,nvar,k);
       CC_til  = CCt_til'; 
       C_sim   = C_til + chol(VC_til)'*rndn(nvar*k,1);       
       CCt_sim = reshape(C_sim,nvar,k);
       CC_sim  = CCt_sim';

       /* Draw sigi_sim
       */
       if zeta == 0;
          H = nobs*Sig_hat + (CC_sim - CC_hat)'*XXXX*(CC_sim - CC_hat);
          Z = chol(inv(H))'*rndn(nvar,nobs);
       else;
          H = nobs*Sig_hat + (CC_sim - CC_hat)'*XXXX*(CC_sim - CC_hat) + H_min;
          Z = chol(inv(H))'*rndn(nvar,nobs+nu_min);
       endif;       


          H = nobs*Sig_hat + (CC_sim - CC_hat)'*XXXX*(CC_sim - CC_hat);
          Z = chol(inv(H))'*rndn(nvar,nobs);
          
       Sigi_sim = Z*Z';
       Sig_sim  = inv(Sigi_sim);

       /* Check for stationarity
       */
       if nlags == 1; 
          CCtilde = CCt_sim[.,2:k];
       else;
          CCtilde =  CCt_sim[.,2:k] |
                 (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
       endif;
       {laCCtil, vCCtil} = eigv(CCtilde);
       if maxc(abs(laCCtil)) < 0.999; 
          val = 1;
       endif;

     endo;

     varparasim[j,.] = C_sim' ~ vech(Sig_sim)';
     
     /* Evaluate likelihood and likelihood*prior at draw
     */
     likesim  = -0.5*nobs*nvar*ln(2*pi) -0.5*nobs*ln(det(Sig_sim))
                -0.5*sumc(diag(Sigi_sim*(YY-XX*CC_sim)'*(YY-XX*CC_sim)));
     if zetind > 0;
        priorsim = -0.5*nvar*k*ln(2*pi) + 0.5*ln(det(VC_mininv*zeta)) 
                   -0.5*(C_sim-C_min)'*(VC_mininv*zeta)*(C_sim-C_min);
/*                   +lpdfiw(Sig_sim,H_min,nu_min);    */
     else;
        priorsim = 0;
     endif;
     statsim[j,.] = (likesim+priorsim) ~ likesim;

     /* H-step ahead forecasting 
     */
     if nlags == 1;
        CCtilde = CCt_sim[.,1+1:k];
        mutilde = CCt_sim[.,1];
     else;
        CCtilde =  CCt_sim[.,1+1:k] |
                 (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
        mutilde =  CCt_sim[.,1] | zeros(nvar*(nlags-1),1) ;
     endif;

     varparah = C_sim';
      
     if hmax > 1;
         hind = 2;
         CCtildeh = CCtilde;
         mutildeh = mutilde;
         
         do until hind > hmax;
         
            CCtildeh = CCtilde*CCtildeh;
            mutildeh = mutilde + CCtildeh*mutildeh;
            CCth_sim = mutildeh[1:nvar,.] ~ CCtildeh[1:nvar,.];
            varparah = varparah ~ vec(CCth_sim')';
         
            hind = hind + 1;
         endo;
         
     endif;
         
     varparahsim[j,.] = varparah;      
      
     j = j+1;
  endo;
  
  if blockind > 2;      
     totaldraws = totaldraws + nsim;
     varparahmean = varparahmean + sumc(varparahsim)';      
  endif;
     
  /* Write Output
  */
  wr = writer(fhvarpara,varparasim);
  wr = writer(fhvarstat,statsim);

  blockind = blockind + 1;

endo;

varparahmean = varparahmean/totaldraws;
varparahmean = reshape(varparahmean,hmax,nvarparah/hmax);

wr = writer(fhvarparah,varparahmean);

closeall fhvarpara, fhvarparah, fhvarstat;
