/***************************************************************/
/*                                                             */
/*     VAR analysis: simulate predictive distribution          */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  vpred.g
** created:   11/07/01
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

k      = 1+nvar*nlags;
lpath  = parapath $+ "\\mhrun" $+ mhrun $+"\\";
opath  = predpath $+ "\\mhrun" $+ mhrun $+"\\";

/* Open files with VAR Parameter Draws
*/
lvarpara = lpath $+ lmodel  $+ llam $+ lsamp;
open fhvarpara = ^lvarpara for read;

load path=^datapath xxpred, nobsfut;

/* Initialize Output files
*/
ody = opath $+ omodel $+ llam $+ lsamp $+ "dy";
oy  = opath $+ omodel $+ llam $+ lsamp $+ "y";

odp = opath $+ omodel $+ llam $+ lsamp $+ "dp";
op  = opath $+ omodel $+ llam $+ lsamp $+ "p";

orr = opath $+ omodel $+ llam $+ lsamp $+ "r";


create fhdy = ^ody with DY, nobsfut, 8;
create fhy  = ^oy  with Y , nobsfut, 8;

create fhdp = ^odp with DP, nobsfut, 8;
create fhp  = ^op  with P , nobsfut, 8;

create fhr  = ^orr  with R , nobsfut, 8;

/* Initialization
*/ 

eofloop = 0;
loopct  = 0;

do until eofloop; 

   /* Read blocks of size nblock
   ** and generate a vector of alphas
   ** of the same dimension
   */
   varparasim = readr( fhvarpara, nblock );

   nread   = rows(varparasim);
    
   DYsim = zeros(nread,nobsfut); 
   Ysim  = zeros(nread,nobsfut);

   DPsim = zeros(nread,nobsfut);
   Psim  = zeros(nread,nobsfut);

   Rsim  = zeros(nread,nobsfut);

   j = 1;
   do until j > nread;

     C_sim   = varparasim[j,1:nvar*k]';
     CCt_sim = reshape(C_sim,nvar,k);
     CC_sim  = CCt_sim';

     Sig_sim = xpnd( varparasim[j,nvar*k+1:cols(varparasim)]' );  
     
     /* Phi_sim = reshape(varidsim[j,.],2,2)'; */

     Sig_chol = chol(Sig_sim)'; /* *Phi_sim;*/
     
     YYsim  = zeros(nobsfut,nvar);
     YYcsim = zeros(nobsfut,nvar);
     xxpredT = xxpred;
     
     YYsim[1,.]  = XXpredT*CC_sim + ( rndn(1,nvar).*shocks )*Sig_chol';
     YYcsim[1,.] = YYsim[1,.];
     if nlags > 1;
        XXpredT[.,1+nvar+1:k]    = XXpredT[.,1+1:k-nvar];
     endif;
     XXpredT[.,1+1:1+nvar]    = YYsim[1,.];
           
     t = 2;
     do until t > nobsfut;
      
         YYsim[t,.] = XXpredT*CC_sim + ( rndn(1,nvar).*shocks )*Sig_chol';
         YYcsim[t,.] = YYcsim[t-1,.] + YYsim[t,.];
         if nlags > 1;
             XXpredT[.,1+nvar+1:k] = XXpredT[.,1+1:k-nvar];
         endif;
         XXpredT[.,1+1:1+nvar]    = YYsim[t,.];
         t = t+1;

     endo;
      
     ysim[j,.]  = YYcsim[.,1]';
     dysim[j,.] = YYsim[.,1]';

     psim[j,.]  = YYcsim[.,2]';
     dpsim[j,.] = YYsim[.,2]';
     
     rsim[j,.] = YYsim[.,3]';
     
     j = j+1;

  endo;

 
  /* Save the results
  */
  wr = writer(fhdy,dysim);
  wr = writer(fhy, ysim);

  wr = writer(fhdp,dpsim);
  wr = writer(fhp,psim);

  wr = writer(fhr,rsim);

  loopct = loopct + 1;

  eofloop = eof(fhvarpara);

endo;

closeall fhdy, fhy, fhdp, fhp, fhr,
         fhvarpara, fhvarid;
