/* filename:    vdforec.g
** description: Convert Simulated Draws into Forecast
** created:     11/05/01
*/

new;
closeall;
library pgraph, user;
cls;

#include c:\projects\active\dsgeprio\Gausscgg\pathspec.g
#include c:\projects\active\dsgeprio\Gausscgg\varspec.g

mhrun = "1";
outstr = outpath $+ "\\varfor" $+ mhrun $+ ".out";

output file = ^outstr reset;
output file = ^outstr off;

/* Specify Input Files
** lmodel: vd[mspec] 
*/
modelid    = ftos(mspec,"%*.*lf",1,0);
lpath      = parapath $+ "\\mhrun" $+ mhrun $+"\\";
lmodel     = "vd" $+ modelid;


/* Specify Output Files
*/
opath = predpath $+ "\\mhrun" $+ mhrun $+ "\\";
omodel = lmodel;

/* Extend lambda grid for lam = 0 (unrestr VAR)
*/
lamgridex = 0 | lamgrid;


hind = 1;
do until hind > hmax;
   /* for each forecasting horizon 
   */

   /* initialize the forecast matrix 
   ** column 1: forecast period
   ** column 2: forecast for different lambdas
   ** last column: forecast for lambdahat
   ** rows: forecast from different origins
   */
   forecast      = zeros(rows(Tgrid),1+nvar*(rows(lamgridex)+1));
   forecasterror = zeros(rows(Tgrid),1+nvar*(rows(lamgridex)+1));
   
   Tind = 1;
   do until Tind > rows(Tgrid);

      T0 = Tgrid[Tind,1];
      Tend = Tgrid[Tind,2];

      datafile = datapath $+ "//data4.txt";

      load data_q[219,4]=^datafile;

      /* Compute growth rates: from 1947:II to 2001:III, 218 observations
      */
      series = 100* ( ln(data_q[2:rows(data_q),1:2])-ln(data_q[1:rows(data_q)-1,1:2]) ) ~
                    ( data_q[2:rows(data_q),3]);
                    
      /* Eliminate Observations for which the Federal Funds Rate is not available
      ** -> 189 observations, starting 1954:III
      */
      series = series[30:rows(series),.];
          
      nobs   = rows(series);
      ti     = seqa(1954.5,0.25,nobs);                    

      YYall     = series[1+T0:nobs,.];
      XXall     = ones(nobs-T0, 1+nvar*nlags);
      ti        = ti[1+T0:nobs,.];

      i = 1;
      do until i > nlags;
         XXall[.,2+(i-1)*nvar:1+i*nvar] = series[T0-i+1:nobs-i,.];
         i = i+1;
      endo;

      /* Index of forecast origin is        Tend-T0
      ** Index of forecasted observation is Tend-T0+hind
      */
      forecast[Tind,1] = ti[Tend-T0]+hind/4;
      if (Tend-T0+hind) LE rows(YYall);
         forecasterror[Tind,1] = ti[Tend-T0]+hind/4;
      endif;
      
      /* load optimal lambda
      ** first row:  index of lamhat
      ** second row: value of lamhat
      */
   
      llamhat = parapath $+ "\\" $+ "mhrun" $+ mhrun $+ "\\" $+ lmodel $+ "ladd";
      open fhlamhat = ^llamhat for read;
      lamindhat = readr(fhlamhat,1)';
      closeall fhlamhat;
      
      lamindex = 1;
      do until lamindex > rows(lamgridex);

        locate 1,1;
        "Forecast Horizon" hind "of" hmax;
        "Sample Iteration" Tind "of" rows(Tgrid);
        "Lambda Iteration" Lamindex "of" rows(lamgridex);

        lambda = lamgridex[lamindex];
        lamind = lamindex - 1;

        /* Load posterior mean of parameter matrix
        */
       
        if lamind < 10;
           llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
        else;
           llam = ftos(lamind,"%*.*lf",2,0);
        endif;

        if Tind < 10;
           lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
        else;
           lsamp = ftos(Tind,"%*.*lf",2,0);
        endif;

        lvarparah  = lpath $+ lmodel $+ llam $+ lsamp $+ "h";

        open fhvarparah = ^lvarparah for read;
        varparah = readr(fhvarparah,hmax);
        closeall fhvarparah;
        
        /* forecasts of output and inflation are cumulative 
        ** -> take cumulative sums of VAR coefficients
        */
        varparahstar = cumsumc(varparah[.,1:2*(1+nvar*nlags)]) ~ varparah[.,2*(1+nvar*nlags)+1:3*(1+nvar*nlags)];
        
        /* Select the row corresponding to the forecast horizon
        */
        CCth = reshape(varparahstar[hind,.]',nvar,1+nvar*nlags);
        CCh  = CCth';
        
        /* Compute the forecast and forecasterror
        */
        yyhhat = XXall[Tend-T0+1,.]*CCh;
        forecast[Tind,1+nvar*(lamindex-1)+1:1+nvar*(lamindex)] = yyhhat;        
        
        if (Tend-T0+hind) LE rows(YYall);
           YYhactual = sumc(YYall[Tend-T0+1:Tend-T0+hind,1:2])' ~ YYall[Tend-T0+hind,3];
           forecasterror[Tind,1+nvar*(lamindex-1)+1:1+nvar*(lamindex)] = YYhactual - YYhhat;
        endif;
        
        lamindex = lamindex+1;

      endo;
      
      /* Calculate forecast/error based on lambdahat
      */
      forecast[Tind,cols(forecast)-nvar+1:cols(forecast)]           
         = forecast[Tind,1+nvar*(1+lamindhat[Tind]-1)+1:1+nvar*(1+lamindhat[Tind]) ];
      forecasterror[Tind,cols(forecast)-nvar+1:cols(forecasterror)] 
         = forecasterror[Tind,1+nvar*(1+lamindhat[Tind]-1)+1:1+nvar*(1+lamindhat[Tind])];
      
      Tind = Tind+1;  

   endo;

   /* save the results for the forecast horizon 
   */
   if hind < 10;
      hstr = "0" $+ ftos(hind,"%*.*lf",1,0);
   else;
      hstr = ftos(hind,"%*.*lf",2,0);
   endif;

   oforecast      = opath $+ "\\" $+ omodel $+ "f" $+ hstr;
   oforecasterror = opath $+ "\\" $+ omodel $+ "e" $+ hstr;

   create fhforecast = ^oforecast with FORECAST, cols(forecast), 8;
   create fhforecasterror = ^oforecasterror with ERROR, cols(forecasterror), 8;

   wr = writer(fhforecast,forecast);
   wr = writer(fhforecasterror,forecasterror);
   
   closeall fhforecast, fhforecasterror;

   hind = hind+1;
endo;

end;
