/* filename:    vforst1.g
** description: compute forecast error statistics
** created:     11/05/01
*/

new;
closeall;
library pgraph, user;
cls;

#include c:\projects\active\dsgeprio\Gausscgg\pathspec.g
#include c:\projects\active\dsgeprio\Gausscgg\varspec.g

mhrun = "1";
outstr = outpath $+ "\\vforst1" $+ mhrun $+ ".out";

output file = ^outstr reset;
output file = ^outstr off;

/* Specify Input Files
*/
modelid    = ftos(mspec,"%*.*lf",1,0);
lpath      = predpath $+ "\\mhrun" $+ mhrun $+"\\";
vdmodel     = "vd" $+ modelid;
vmmodel     = "vm";

/* Extend lambda grid for lam = 0 (unrestr VAR)
*/
lamgridex = 0 | zetgrid;
zetgridex = 0 | zetgrid;

vderrordim = nvar*(rows(lamgridex)+1);
vmerrordim = nvar*(rows(zetgridex)+1);


/* Initialize Output matrices
*/
forec_y   = zeros(hmax,4);
forec_pi  = zeros(hmax,4);
forec_R   = zeros(hmax,4);
forec_all = zeros(hmax,4);

forecvd_y   = zeros(hmax,rows(lamgridex));
forecvd_pi  = zeros(hmax,rows(lamgridex));
forecvd_R   = zeros(hmax,rows(lamgridex));
forecvd_all = zeros(hmax,rows(lamgridex));

hind = 1;
do until hind > hmax;
   /* for each forecasting horizon 
   */

   if hind < 10;
      hstr = "0" $+ ftos(hind,"%*.*lf",1,0);
   else;
      hstr = ftos(hind,"%*.*lf",2,0);
   endif;

   lvderror = lpath $+ "\\" $+ vdmodel $+ "e" $+ hstr;
   lvmerror = lpath $+ "\\" $+ vmmodel $+ "e" $+ hstr;

   open fhvderror = ^lvderror for read;
   open fhvmerror = ^lvmerror for read;

   vderror = readr(fhvderror, Tgrid_n);
   vmerror = readr(fhvmerror, Tgrid_n);
 
   closeall fhvderror, fhvmerror;

   /* Delete invalid entries
   */
   vderror = delif(vderror, (vderror[.,1] .EQ 0));
   vmerror = delif(vmerror, (vmerror[.,1] .EQ 0));

   /* Compute RMSE's
   */
   vdrmse = sqrt(meanc( vderror[.,2:vderrordim+1]^2 )');
   vmrmse = sqrt(meanc( vmerror[.,2:vmerrordim+1]^2 )');
 
   /* Compute percentage RMSE reductions
   */
   vdrelrmse = vdrmse;
   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdrelrmse[1,nvar*(lind-1)+1:nvar*lind]
         = -100*(vdrmse[1,nvar*(lind-1)+1:nvar*lind] ./ vdrmse[1,1:nvar] - 1);
      lind = lind+1;
   endo;

   vmrelrmse = vmrmse;
   lind = 1;
   do until lind > rows(zetgridex)+1;
      vmrelrmse[1,nvar*(lind-1)+1:nvar*lind]
         = -100*( vmrmse[1,nvar*(lind-1)+1:nvar*lind] ./ vmrmse[1,1:nvar] - 1 );
      lind = lind+1;
   endo;
      
   /* Compute log-determinant criterion
   */
   vdlndet = zeros(1,rows(lamgridex)+1);
   vmlndet = zeros(1,rows(zetgridex)+1);

   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdlndet[lind] 
         = -ln(det( vderror[.,1+nvar*(lind-1)+1:1+nvar*lind]'*vderror[.,1+nvar*(lind-1)+1:1+nvar*lind]/Tgrid_n ))/(2*nvar);
      lind = lind+1;
   endo;

   lind = 1;
   do until lind > rows(zetgridex)+1;
      vmlndet[lind] 
         = -ln(det( vmerror[.,1+nvar*(lind-1)+1:1+nvar*lind]'*vmerror[.,1+nvar*(lind-1)+1:1+nvar*lind]/Tgrid_n ))/(2*nvar);
      lind = lind+1;
   endo;

   /* Compute relative improvement in log-determinant criterion
   */
   vdrellndet = vdlndet;
   vmrellndet = vmlndet;

   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdrellndet[1,lind]
         = 100*(vdlndet[1,lind]-vdlndet[1,1]);
      lind = lind+1;
   endo;

   lind = 1;
   do until lind > rows(zetgridex)+1;
      vmrellndet[1,lind]
         = 100*(vmlndet[1,lind]-vmlndet[1,1]);
      lind = lind+1;
   endo;

   /* Save Results in output matrices 
   */
   forec_y[hind,1] = hind;
   forec_y[hind,2] = vdrmse[1];
   forec_y[hind,3] = vdrelrmse[rows(lamgridex)*nvar+1];
   forec_y[hind,4] = vmrelrmse[rows(lamgridex)*nvar+1];

   forec_pi[hind,1] = hind;
   forec_pi[hind,2] = vdrmse[2];
   forec_pi[hind,3] = vdrelrmse[rows(lamgridex)*nvar+2];
   forec_pi[hind,4] = vmrelrmse[rows(lamgridex)*nvar+2];

   forec_R[hind,1] = hind;
   forec_R[hind,2] = vdrmse[3];
   forec_R[hind,3] = vdrelrmse[rows(lamgridex)*nvar+3];
   forec_R[hind,4] = vmrelrmse[rows(lamgridex)*nvar+3];

   forec_all[hind,1] = hind;
   forec_all[hind,2] = vdlndet[1];
   forec_all[hind,3] = vdrellndet[rows(lamgridex)+1];
   forec_all[hind,4] = vmrellndet[rows(lamgridex)+1];

   vdrelrmsemat = reshape(vdrelrmse,rows(lamgridex)+1,nvar);   

   forecvd_y[hind,.]    = vdrelrmsemat[1:rows(lamgridex),1]';
   forecvd_pi[hind,.]   = vdrelrmsemat[1:rows(lamgridex),2]';
   forecvd_R[hind,.]    = vdrelrmsemat[1:rows(lamgridex),3]';
   forecvd_all[hind,.]  = vdrellndet[.,1:rows(lamgridex)];

   /* Output 
   */
   "Horizon" hind;
   "-----------------------";
   "";
   
   "RMSE's - DSGE Prior";
   reshape(vdrmse,rows(lamgridex)+1,nvar);
   "";
   
   "Relative RMSE's - DSGE Prior";
   reshape(vdrelrmse,rows(lamgridex)+1,nvar);

   "RMSE's - Minn Prior";
   reshape(vmrmse,rows(zetgridex)+1,nvar);
   "";
   
   "Relative RMSE's - Minn Prior";
   reshape(vmrelrmse,rows(zetgridex)+1,nvar);
   "";

   "LnDet Criterion - DSGE Prior";
   vdlndet;
   "";

   "Relative LnDet - DSGE Prior";
   vdrellndet;
   "";

   "LnDet Criterion - Minn Prior";
   vmlndet;
   "";

   "Relative LnDet - Minn Prior";
   vmrellndet;
   "";
   "====================================";
   "";
   "";

   hind = hind+1;

endo;


"Summary Statistics";
"------------------";
"";
"Output Forecasts";
forec_y;
"";
"Inflation Forecasts";
forec_pi;
"";
"Interest Rate Forecasts";
forec_R;
"";
"ln-det Statistic";
forec_all;

/* Generate Plot with forecast performance as function
** of lambda
*/

graphset;
begwind;
window(2,2,1);

title("Output");
surface(lamgridex',seqa(1,1,hmax),forecvd_y);
nextwind;

title("Inflation");
surface(lamgridex',seqa(1,1,hmax),forecvd_pi);
nextwind;

title("Interest Rates");
surface(lamgridex',seqa(1,1,hmax),forecvd_R);
nextwind;

title("LnDet Criterion");
surface(lamgridex',seqa(1,1,hmax),forecvd_all);
endwind;
