/* filename:    vforst2.g
** description: compute forecast error statistics
** created:     11/05/01
*/

new;
closeall;
library pgraph, user;
cls;

format /mat /on /mb1 /ros 8,4;

#include c:\projects\active\dsgeprio\Gausscgg\pathspec.g
#include c:\projects\active\dsgeprio\Gausscgg\varspec.g

mhrun  = "2";
mhrunp = "2p";
nforc  = 80;
fmin   = 1;
fmax   = 80;

outstr = outpath $+ "\\vforst2" $+ mhrun $+ ".out";

output file = ^outstr reset;
output file = ^outstr off;

/* Specify Input Files
*/
modelid    = ftos(mspec,"%*.*lf",1,0);
lpath      = predpath $+ "\\mhrun" $+ mhrun $+"\\";
lpathp     = predpath $+ "\\mhrun" $+ mhrunp $+"\\";
vdmodel     = "vd" $+ modelid;

/* Extend lambda grid for lam = 0 (unrestr VAR)
*/
lamgridex = 0 | lamgrid;

vderrordim  = nvar*(rows(lamgridex)+1);
vdperrordim = nvar*(rows(lamgridex)+1);

/* Initialize Output matrices
*/
forec_y   = zeros(hmax,4);
forec_pi  = zeros(hmax,4);
forec_R   = zeros(hmax,4);
forec_all = zeros(hmax,4);

forecvd_y   = zeros(hmax,rows(lamgridex));
forecvd_pi  = zeros(hmax,rows(lamgridex));
forecvd_R   = zeros(hmax,rows(lamgridex));
forecvd_all = zeros(hmax,rows(lamgridex));

forecvdp_y   = zeros(hmax,rows(lamgridex));
forecvdp_pi  = zeros(hmax,rows(lamgridex));
forecvdp_R   = zeros(hmax,rows(lamgridex));
forecvdp_all = zeros(hmax,rows(lamgridex));


hind = 1;
do until hind > hmax;

   /* for each forecasting horizon 
   */

   if hind < 10;
      hstr = "0" $+ ftos(hind,"%*.*lf",1,0);
   else;
      hstr = ftos(hind,"%*.*lf",2,0);
   endif;

   lvderror  = lpath  $+ "\\" $+ vdmodel $+ "e" $+ hstr;
   lvdperror = lpathp $+ "\\" $+ vdmodel $+ "e" $+ hstr;

   open fhvderror  = ^lvderror for read;
   open fhvdperror = ^lvdperror for read;

   vderror  = readr(fhvderror,  nforc);
   vdperror = readr(fhvdperror, nforc);
 
   closeall fhvderror, fhvdperror;

   /* Delete invalid entries
   */
   vderror  = delif(vderror,  (vderror[.,1]  .EQ 0));
   vdperror = delif(vdperror, (vdperror[.,1] .EQ 0));

   /* Compute RMSE's
   */
   vdrmse  = sqrt(meanc( vderror[fmin:fmax,2:vderrordim+1]^2 )');
   vdprmse = sqrt(meanc( vdperror[fmin:fmax,2:vdperrordim+1]^2 )');
 
   /* Compute percentage RMSE reductions
   */
   vdrelrmse = vdrmse;
   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdrelrmse[1,nvar*(lind-1)+1:nvar*lind]
         = -100*(vdrmse[1,nvar*(lind-1)+1:nvar*lind] ./ vdrmse[1,1:nvar] - 1);
      lind = lind+1;
   endo;

   vdprelrmse = vdprmse;
   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdprelrmse[1,nvar*(lind-1)+1:nvar*lind]
         = -100*( vdprmse[1,nvar*(lind-1)+1:nvar*lind] ./ vdprmse[1,1:nvar] - 1 );
      lind = lind+1;
   endo;
      
   /* Compute log-determinant criterion
   */
   vdlndet  = zeros(1,rows(lamgridex)+1);
   vdplndet = zeros(1,rows(lamgridex)+1);

   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdlndet[lind] 
         = (ln(det( vderror[fmin:fmax,1+nvar*(lind-1)+1:1+nvar*lind]'*vderror[fmin:fmax,1+nvar*(lind-1)+1:1+nvar*lind]/(fmax-fmin+1) )));
      lind = lind+1;
   endo;

   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdplndet[lind] 
         = (ln(det( vdperror[fmin:fmax,1+nvar*(lind-1)+1:1+nvar*lind]'*vdperror[fmin:fmax,1+nvar*(lind-1)+1:1+nvar*lind]/(fmax-fmin+1) )));
      lind = lind+1;
   endo;

   /* Compute relative improvement in log-determinant criterion
   */
   vdrellndet  = vdlndet;
   vdprellndet = vdplndet;

   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdrellndet[1,lind]
         = -100*(vdlndet[1,lind]-vdlndet[1,1])/(2*nvar);
      lind = lind+1;
   endo;

   lind = 1;
   do until lind > rows(lamgridex)+1;
      vdprellndet[1,lind]
         = -100*(vdplndet[1,lind]-vdplndet[1,1])/(2*nvar);
      lind = lind+1;
   endo;

   /* Save Results in output matrices 
   */
   forec_y[hind,1] = hind;
   forec_y[hind,2] = vdrmse[1];
   forec_y[hind,3] = vdrelrmse[rows(lamgridex)*nvar+1];
   forec_y[hind,4] = vdprelrmse[rows(lamgridex)*nvar+1];

   forec_pi[hind,1] = hind;
   forec_pi[hind,2] = vdrmse[2];
   forec_pi[hind,3] = vdrelrmse[rows(lamgridex)*nvar+2];
   forec_pi[hind,4] = vdprelrmse[rows(lamgridex)*nvar+2];

   forec_R[hind,1] = hind;
   forec_R[hind,2] = vdrmse[3];
   forec_R[hind,3] = vdrelrmse[rows(lamgridex)*nvar+3];
   forec_R[hind,4] = vdprelrmse[rows(lamgridex)*nvar+3];

   forec_all[hind,1] = hind;
   forec_all[hind,2] = vdlndet[1];
   forec_all[hind,3] = vdrellndet[rows(lamgridex)+1];
   forec_all[hind,4] = vdprellndet[rows(lamgridex)+1];

   vdrelrmsemat = reshape(vdrelrmse,rows(lamgridex)+1,nvar);   

   forecvd_y[hind,.]    = vdrelrmsemat[1:rows(lamgridex),1]';
   forecvd_pi[hind,.]   = vdrelrmsemat[1:rows(lamgridex),2]';
   forecvd_R[hind,.]    = vdrelrmsemat[1:rows(lamgridex),3]';
   forecvd_all[hind,.]  = vdrellndet[.,1:rows(lamgridex)];

   vdprelrmsemat = reshape(vdprelrmse,rows(lamgridex)+1,nvar);   

   forecvdp_y[hind,.]    = vdprelrmsemat[1:rows(lamgridex),1]';
   forecvdp_pi[hind,.]   = vdprelrmsemat[1:rows(lamgridex),2]';
   forecvdp_R[hind,.]    = vdprelrmsemat[1:rows(lamgridex),3]';
   forecvdp_all[hind,.]  = vdprellndet[.,1:rows(lamgridex)];


   /* Output 
   */
   "Horizon" hind;
   "-----------------------";
   "";
   
   "RMSE's - VAR(DSGE), Baseline";
   reshape(vdrmse,rows(lamgridex)+1,nvar);
   "";
   
   "Relative RMSE's - VAR(DSGE), Baseline";
   reshape(vdrelrmse,rows(lamgridex)+1,nvar);

   "RMSE's - VAR(DSGE), Policy Experiment";
   reshape(vdprmse,rows(lamgridex)+1,nvar);
   "";
   
   "Relative RMSE's - VAR(DSGE), Policy Experiment";
   reshape(vdprelrmse,rows(lamgridex)+1,nvar);
   "";

   "LnDet Criterion - VAR(DSGE), Baseline";
   vdlndet;
   "";

   "Relative LnDet - VAR (DSGE), Baseline";
   vdrellndet;
   "";

   "LnDet Criterion - VAR(DSGE), Policy Experiment";
   vdplndet;
   "";

   "Relative LnDet - VAR(DSGE), Policy Experiment";
   vdprellndet;
   "";
   "====================================";
   "";
   "";

   hind = hind+1;

endo;


"Summary Statistics";
"------------------";
"";
"Output Forecasts";
forec_y;
"";
"Inflation Forecasts";
forec_pi;
"";
"Interest Rate Forecasts";
forec_R;
"";
"ln-det Statistic";
forec_all;

/* Generate Plot with forecast performance as function
** of lambda
*/

graphset;
begwind;
window(2,2,1);

title("Output - Baseline");
surface(lamgridex',seqa(1,1,hmax),forecvd_y);
nextwind;

title("Inflation - Baseline");
surface(lamgridex',seqa(1,1,hmax),forecvd_pi);
nextwind;

title("Interest Rates - Baseline");
surface(lamgridex',seqa(1,1,hmax),forecvd_R);
nextwind;

title("LnDet Criterion - Baseline");
surface(lamgridex',seqa(1,1,hmax),forecvd_all);
endwind;

graphset;
begwind;
window(2,2,1);

title("Output - Policy Experiment");
surface(lamgridex',seqa(1,1,hmax),forecvdp_y);
nextwind;

title("Inflation - Policy Experiment");
surface(lamgridex',seqa(1,1,hmax),forecvdp_pi);
nextwind;

title("Interest Rates - Policy Experiment");
surface(lamgridex',seqa(1,1,hmax),forecvdp_R);
nextwind;

title("LnDet Criterion - Policy Experiment");
surface(lamgridex',seqa(1,1,hmax),forecvdp_all);
endwind;
