/* filename:    vprdpmom.g
** description: Compute posterior mean and std and CI based on
**              output of posterior simulator
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include c:\projects\active\dsgeprio\gausscgg\pathspec.g;
#include c:\projects\active\dsgeprio\gausscgg\varspec.g;

nburn       = 1;         /* Number of initial draws to be discarded */
hpdprob     = 0.5 | 0.9 ;
nblock      = 1000;
mhrun       = "1";

modelid = ftos(mspec,"%*.*lf",1,0);
lmodel  = "v" $+ modelid;
omodel  = "v" $+ modelid;


predid  = "y" $| "p" $| "r";

lamgridex = 0 | lamgrid;

Tind = 1;
do until Tind > rows(Tgrid);

   lamindex = 1;
   do until lamindex > rows(lamgridex);

      lamind = lamindex - 1;

      predind = 1;
      do until predind > rows(predid);


         if lamind < 10;
          llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
         else;
          llam = ftos(lamind,"%*.*lf",2,0);
         endif;

         if Tind < 10;
          lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
         else;
          lsamp = ftos(Tind,"%*.*lf",2,0);
         endif;
      
         lpath  = predpath $+ "\\mhrun" $+ mhrun $+ "\\";
         opath  = predpath $+ "\\mhrun" $+ mhrun $+ "\\";

         ldraw  = lpath $+ lmodel $+ llam $+ lsamp $+ predid[predind];
         odraws = opath $+ lmodel $+ llam $+ lsamp $+ predid[predind]  $+ "s";

         locate 1,1;
         "Lambda    " lamindex "out of" rows(lamgridex);
         "Sample    " Tind   "out of" rows(Tgrid); 
         "Variable  " $predid[predind];
         
         gosub computmom;
         
         predind = predind + 1;
      endo;
      lamindex = lamindex + 1;
   endo;
   Tind = Tind+1;
endo;

end;
         
/***********************************************************
**  S U B R O U T I N E
*/
computmom:

open fhdraw = ^ldraw for read;

drawrow = readr( fhdraw, nburn); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, nburn);

/* Part 1: Compute the mean of x(i) and x^2(i)
*/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawcross  = zeros(drawdim,drawdim);

eofloop = 0;
ndraws  = 0;

do until eofloop; 

   drawblock = readr( fhdraw, nblock );

   drawmean  = drawmean    + sumc(drawblock)';
   ndraws = ndraws + rows(drawblock);

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;

/* Part 2: HPD Interval 1
*/
drawrow     = seekr( fhdraw, nburn); 
drawci1     = zeros(2,drawdim);

j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, nburn); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, nblock );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);
         
   endo;
   
   drawci1[.,j] = hpdint(drawcol,hpdprob[1]);
   j = j+1;

endo;
clear drawcol;

/* Part 3: HPD Interval 2
*/
drawrow     = seekr( fhdraw, nburn); 
drawci2     = zeros(2,drawdim);

j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, nburn); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, nblock );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);
         
   endo;
   
   drawci2[.,j] = hpdint(drawcol,hpdprob[2]);
   j = j+1;

endo;
clear drawcol;



create fhdraws = ^odraws with DRAWSUM, drawdim, 8;
wr=writer(fhdraws, real(drawmean | drawci1 | drawci2) );
closeall fhdraws; 

return;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;
