/* filename:    vprdpsd.g
** description: Computes standard dev of interest, inflation and output growth trajectories
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include c:\projects\active\dsgeprio\gausscgg\pathspec.g;
#include c:\projects\active\dsgeprio\gausscgg\varspec.g;

nburn   = 1;         /* Number of initial draws to be discarded */
nblock  = 1000;
ngrid   = 100;
mhrun   = "1";

/* traj 1 =
** 1  : entire trajectory
** 14 : post 1982 trajectory
*/
traj1   = 14;

modelid = ftos(mspec,"%*.*lf",1,0);
lmodel  = "v" $+ modelid;
omodel  = "v" $+ modelid;

predid  = "dy" $| "dp" $| "r" ;
predido = "yv" $| "pv" $| "rv";

dysdgridmin = 0;
dysdgridmax = 2.5;
dysdgrid    = seqa(dysdgridmin,(dysdgridmax - dysdgridmin)/(ngrid-1),ngrid);

dpsdgridmin = 0;
dpsdgridmax = 2;
dpsdgrid    = seqa(dpsdgridmin,(dpsdgridmax - dpsdgridmin)/(ngrid-1),ngrid);

rsdgridmin = 0;
rsdgridmax = 10;
rsdgrid    = seqa(rsdgridmin,(rsdgridmax - rsdgridmin)/(ngrid-1),ngrid);


sdgrid_all  = dysdgrid ~ dpsdgrid ~ rsdgrid;

lamgridex = 0 | lamgrid;

Tind = 1;
do until Tind > rows(Tgrid);

   lamindex = 1;
   do until lamindex > rows(lamgridex);

      lamind = lamindex - 1;

      predind = 1;
      do until predind > rows(predid);

         if lamind < 10;
          llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
         else;
          llam = ftos(lamind,"%*.*lf",2,0);
         endif;

         if Tind < 10;
          lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
         else;
          lsamp = ftos(Tind,"%*.*lf",2,0);
         endif;
      
         lpath  = predpath $+ "\\mhrun" $+ mhrun $+ "\\";
         opath  = predpath $+ "\\mhrun" $+ mhrun $+ "\\";

         ldraw  = lpath $+ lmodel $+ llam $+ lsamp $+ predid[predind];
         odraws = opath $+ lmodel $+ llam $+ lsamp $+ predido[predind];

         locate 1,1;
         "Lambda    " lamindex  "out of" rows(lamgridex);
         "Sample    " Tind      "out of" rows(Tgrid);
         "Variable  " $predid[predind];

         sdgrid = sdgrid_all[.,predind];
         
         gosub sddens;
         
         predind = predind + 1;
      endo;

      lamindex = lamindex + 1;
   endo;
   Tind = Tind+1;
endo;

end;
         
/***********************************************************
**  S U B R O U T I N E
*/
sddens:

open fhdraw = ^ldraw for read;

drawrow = seekr( fhdraw, nburn);
drawrow = readr( fhdraw, 1); 
drawdim = cols(drawrow);

/* Compute Standard Dev for each trajectory
*/
sdtraj  = stdc(drawrow[.,traj1:cols(drawrow)]');

eofloop = 0;
ndraws  = 1;

do until eofloop; 

   drawblock = readr( fhdraw, nblock );
   sdtraj  = sdtraj | stdc(drawblock');

   ndraws = ndraws + rows(drawblock);

   eofloop = eof(fhdraw);

endo;

/* Compute Density Estimate for Standard Dev
*/
bwidth    = 1.06*ndraws^(-1/5)*stdc(sdtraj);
pdfsdtraj = zeros(ngrid,1);  

sdind = 1;
do until sdind > ngrid;
   pdfsdtraj[sdind] = 1/(ndraws*bwidth)*sumc( pdfn( (ones(ndraws,1)*sdgrid[sdind]-sdtraj)/bwidth ) );
   sdind = sdind+1;
endo;

create fhdraws = ^odraws with SD, ngrid, 8;
wr=writer(fhdraws, sdgrid' | pdfsdtraj' );
closeall fhdraws; 

return;

